# coding: utf-8
import os
from pprint import pprint
from wechatarticles import ArticlesAPI
from wechatarticles.utils import save_json

if __name__ == "__main__":
    # 该API慎用，不再维护

    # 利用公众号获取链接，并获取阅读点赞
    official_cookie = "official_cookie"
    token = "token"
    appmsg_token = "appmsg_token"
    wechat_cookie = "wechat_cookie"

    nickname = "nickname"

    # 手动输入所有参数
    aa = ArticlesAPI(
        official_cookie=official_cookie,
        token=token,
        appmsg_token=appmsg_token,
        wechat_cookie=wechat_cookie,
    )

    # 自定义爬取，每次爬取5篇以上
    data = aa.complete_info(nickname=nickname, begin="0")
    print(len(data))
    pprint(data)

    # 自定义从某部分开始爬取，持续爬取，直至爬取失败为止，一次性最多爬取40篇（功能未测试，欢迎尝试）
    datas = aa.continue_info(nickname=nickname, begin="0")

    save_json("{}.json".format(nickname), data)



# coding: utf-8
"""
辅助脚本函数
"""
import base64
import html
import json
import os
import re
import time

import requests

from .ArticlesUrls import PC

base_columns = ["url", "title", "date", "headlines"]
custom_columns = ["copyright_stat"]
A_columns = ["read_num", "old_like_num", "like_num"]
B_columns = ["comments_num", "comments_content", "comments_like_num"]
C_columns = ["text_content", "content_num", "pic_num"]
mode_columns = {
    1: A_columns,
    2: B_columns,
    3: C_columns,
    4: A_columns + B_columns,
    5: A_columns + C_columns,
    6: B_columns + C_columns,
    7: A_columns + B_columns + C_columns,
}


# url, readnum likenum
def flatten(x):
    return [y for l in x for y in flatten(l)] if type(x) is list else [x]


def remove_duplicate_json(fname):
    # 删除json中重复的数据
    # fname: xxx.json
    with open(fname, "r", encoding="utf-8") as f:
        data = f.readlines()

    id_re = re.compile(r'datetime": (.+), "fakeid"')
    sort_func = lambda line: id_re.findall(line)[0]

    list_data = list(set(data))
    sort_data = sorted(list_data, key=sort_func)[::-1]

    # sort_data = sorted(list(set_data),
    #                    key=lambda line: re.findall(
    #                        r'datetime": (.+), "fakeid"', line)[0])[::-1]

    with open(fname, "w", encoding="utf-8") as f:
        f.writelines(sort_data)


def end_func(timestamp, end_timestamp):
    if timestamp < end_timestamp:
        print(timestamp, end_timestamp)
        return True
    return False


def transfer_url(url):
    url = html.unescape(html.unescape(url))
    return eval(repr(url).replace("\\", ""))


def save_f(fname):
    i = 1
    while True:
        if os.path.isfile("{}.json".format(fname)):
            i += 1
            fname += "-" + str(i)
        else:
            break

    return fname


# verify_lst = ["mp.weixin.qq.com", "__biz", "mid", "sn", "idx"]
verify_lst = ["mp.weixin.qq.com", "__biz", "mid", "idx"]


def verify_url(article_url):
    for string in verify_lst:
        if string not in article_url:
            return False
    return True


def copyright_num(copyright_stat):
    if copyright_stat == 11:
        return 1  # 标记原创
    else:
        return 0


def copyright_num_detailed(copyright_stat):
    copyright_stat_lst = [14, 12, 201]
    if copyright_stat == 11:
        return 1  # 标记原创
    elif copyright_stat == 100:
        return 0  # 荐号
    elif copyright_stat == 101:
        return 2  # 转发
    elif copyright_stat == 0:
        return 3  # 来源非微信文章
    elif copyright_stat == 1:
        return 4  # 形容词（xxx的公众号）
    elif copyright_stat in copyright_stat_lst:
        return 5
    else:
        return None


def read_nickname(fname):
    # 读取数据
    with open(fname, "r", encoding="utf-8") as f:
        haved_data = f.readlines()
    return [line.split(", ") for line in haved_data]


def get_history_urls(
    biz,
    uin,
    key,
    lst=[],
    start_timestamp=0,
    start_count=10,
    end_count=99999,
    return_flag=False,
):
    """
    获取历史文章链接

    Parameters
    ----------
    biz: str
        公众号id
    uin: str
        个人微信号id
    key: str
        个人微信号key
    lst: list
        已有的数据列表
    start_timestampe: int
        截至时间戳
    start_count: int
        开始的条数
    end_count: int
        截至条数
    return_flag: bool
        是否返回状态信息

    Returns
    -------
    lst:
        获取到的历史文章数据
    """
    t = PC(biz=biz, uin=uin, cookie="")
    flag = True
    try:
        while True:
            res = t.get_urls(key, offset=start_count)
            if res == []:
                break
            start_count += 10
            lst.append(res)
            dt = res[-1]["comm_msg_info"]["datetime"]
            print(start_count, timestamp2date(dt))
            if dt <= start_timestamp or start_count >= end_count:
                break
            time.sleep(5)
    except KeyboardInterrupt as e:
        flag = False
        print("程序手动中断")
    except Exception as e:
        print(e)
        flag = False
        print("获取文章链接失败。。。退出程序")
    finally:
        if return_flag:
            return flag, lst
        return lst


def swap_biz_id(biz=None, fakeid=None):
    if biz == None:
        return str(base64.b64encode(fakeid.encode()), encoding="utf-8")
    if fakeid == None:
        return str(base64.b64decode(biz.encode()), encoding="utf-8")
    return None


# 一些tools，如时间戳转换
def timestamp2date(timestamp):
    """
    时间戳转换为日期

    Parameters
    ----------
    timestamp: int or str
        用户账号

    Returns
    -------
    datetime:
        转换好的日期：年-月-日 时:分:秒
    """
    time_array = time.localtime(int(timestamp))
    datetime = time.strftime("%Y-%m-%d %H:%M:%S", time_array)
    return datetime


def save_json(fname, data):
    """
    保存数据为txt格式

    Parameters
    ----------
    fname: str
        保存为txt文件名
    data: list
        爬取到的数据
    Returns
    -------
    None
    """
    assert isinstance(fname, str)

    if ".json" not in fname:
        raise IOError("fname must be json", fname)
    with open(fname, "a+") as f:
        for item in data:
            f.write(json.dumps(item))
            f.write("\n")



# coding:  utf-8
import hashlib
import os
import time

import requests
from requests.cookies import cookielib

# TODO: 抽象一个基类


class PublicAccountsWeb(object):
    """通过微信公众号网页版抓取链接，或者公众号信息"""

    def __init__(self, cookie, token, proxies={"http": None, "https": None}):
        """
        Parameters
        ----------
        token : str
            登录微信公众号平台之后获取的token
        cookie : str
            登录微信公众号平台之后获取的cookie
        """
        self.s = requests.session()
        self.headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/62.0.3202.62 Safari/537.36"
        }
        self.params = {
            "lang": "zh_CN",
            "f": "json",
        }

        # 手动输入cookie和token登录
        self.__verify_str(cookie, "cookie")
        self.__verify_str(token, "token")
        self.headers["Cookie"] = cookie
        self.params["token"] = token
        self.proxies = proxies

    def __verify_str(self, input_string, param_name):
        """
        验证输入是否为字符串

        Parameters
        ----------
        input_string: str
            输入
        param_name: str
            需要验证的参数名
        """
        if not isinstance(input_string, str):
            raise TypeError("{} must be an instance of str".format(param_name))

    def __save_login_qrcode(self, img):
        """
        存储和显示登录二维码

        Parameters
        ----------
        img: str
            获取到的二维码数据
        """
        import matplotlib.pyplot as plt
        from PIL import Image

        # 存储二维码
        with open("login.png", "wb+") as fp:
            fp.write(img.content)
        # 显示二维码， 这里使用plt的原因是： 等待用户扫描完之后手动关闭窗口继续运行；否则会直接运行
        try:
            img = Image.open("login.png")
        except Exception:
            raise TypeError(u"账号密码输入错误，请重新输入")
        plt.figure()
        plt.imshow(img)
        plt.show()

    def __save_cookie(self, username):
        """
        存储cookies, username用于文件命名

        Parameters
        ----------
        username: str
            用户账号
        """
        # 实例化一个LWPcookiejar对象
        new_cookie_jar = cookielib.LWPCookieJar(username + ".txt")

        # 将转换成字典格式的RequestsCookieJar（这里我用字典推导手动转的）保存到LWPcookiejar中
        requests.utils.cookiejar_from_dict(
            {c.name: c.value for c in self.s.cookies}, new_cookie_jar
        )

        # 保存到本地文件
        new_cookie_jar.save(
            "cookies/" + username + ".txt", ignore_discard=True, ignore_expires=True
        )

    def __read_cookie(self, username):
        """
        读取cookies, username用于文件命名

        Parameters
        ----------
        username: str
            用户账号
        """
        # 实例化一个LWPCookieJar对象
        load_cookiejar = cookielib.LWPCookieJar()
        # 从文件中加载cookies(LWP格式)
        load_cookiejar.load(
            "cookies/" + username + ".txt", ignore_discard=True, ignore_expires=True
        )
        # 工具方法转换成字典
        load_cookies = requests.utils.dict_from_cookiejar(load_cookiejar)
        # 工具方法将字典转换成RequestsCookieJar，赋值给session的cookies.
        self.s.cookies = requests.utils.cookiejar_from_dict(load_cookies)

    def __md5_passwd(self, password):
        """
        微信公众号的登录密码需要用md5方式进行加密

        Parameters
        ----------
        password: str
            加密前的字符串

        Returns
        -------
        str：
            加密后的字符串
        """
        m5 = hashlib.md5()
        m5.update(password.encode("utf-8"))
        pwd = m5.hexdigest()
        return pwd

    def __startlogin_official(self, username, password):
        """
        获取登录二维码，进而获取Cookies

        Parameters
        ----------
        username: str
            用户账号
        password: str
            用户密码

        Returns
        -------
            None
        """
        # 进行md5加密，一些post的参数
        pwd = self.__md5_passwd(password)
        data = {
            "username": username,
            "userlang": "zh_CN",
            "token": "",
            "pwd": pwd,
            "lang": "zh_CN",
            "imgcode": "",
            "f": "json",
            "ajax": "1",
        }

        # 增加headers的keys
        self.headers["Host"] = "mp.weixin.qq.com"
        self.headers["Origin"] = "https://mp.weixin.qq.com"
        self.headers["Referer"] = "https://mp.weixin.qq.com/"

        # 账号密码post的url
        bizlogin_url = "https://mp.weixin.qq.com/cgi-bin/bizlogin?action=startlogin"
        # 获取二维码的url
        qrcode_url = "https://mp.weixin.qq.com/cgi-bin/loginqrcode?action=getqrcode&param=4300&rd=928"

        # 账号密码登录，获取二维码，等待用户扫描二维码，需手动关闭二维码窗口
        self.s.post(bizlogin_url, headers=self.headers, data=data)
        img = self.s.get(qrcode_url)
        self.__save_login_qrcode(img)

        # 去除之后不用的headers的key
        self.headers.pop("Host")
        self.headers.pop("Origin")
        # 获取token
        self.__login_official(username, password)

    def __login_official(self, username, password):
        """
        登录微信公众号平台，获取token

        Parameters
        ----------
        username: str
            用户账号
        password: str
            用户密码
        """
        # 设定headers的referer的请求
        referer = "https://mp.weixin.qq.com/cgi-bin/bizlogin?action=validate&lang=zh_CN&account={}".format(
            username
        )
        self.headers["Referer"] = referer

        # 获取token的data
        data = {
            "userlang": "zh_CN",
            "token": "",
            "lang": "zh_CN",
            "f": "json",
            "ajax": "1",
        }
        # 获取token的url
        bizlogin_url = "https://mp.weixin.qq.com/cgi-bin/bizlogin?action=login"
        res = self.s.post(
            bizlogin_url, data=data, headers=self.headers, proxies=self.proxies
        ).json()

        try:
            # 截取字符串中的token参数
            token = res["redirect_url"].split("=")[-1]
            self.params["token"] = token
            # self.__save_cookie(username)
            self.headers.pop("Referer")
        except Exception:
            # 获取token失败，重新扫码登录
            print("please try again")
            self.__startlogin_official(username, password)

    def official_info(self, nickname, begin=0, count=5):
        """
        根据关键词返回相关公众号的信息

        Parameters
        ----------
        nickname : str
            需要爬取公众号名称
        begin: str or int
            起始爬取的页数
        count: str or int
            每次爬取的数量，1-5

        Returns
        -------
        list:
            相关公众号的对应信息::

                [
                    {
                    'alias': 公众号别名,
                    'fakeid': 公众号唯一id,
                    'nickname': 公众号名称,
                    'round_head_img': 公众号头像的url,
                    'service_type': 1公众号性质
                    },
                ...
                ]
        """
        self.__verify_str(nickname, "nickname")
        # 搜索公众号的url
        search_url = "https://mp.weixin.qq.com/cgi-bin/searchbiz"

        # 增加/更改请求参数
        params = {
            "query": nickname,
            "count": str(count),
            "action": "search_biz",
            "ajax": "1",
            "begin": str(begin),
        }
        self.params.update(params)

        try:
            # 返回与输入公众号名称最接近的公众号信息
            official = self.s.get(
                search_url,
                headers=self.headers,
                params=self.params,
                proxies=self.proxies,
            )
            return official.json()["list"]
        except Exception:
            raise Exception(u"公众号名称错误或cookie、token错误，请重新输入")

    def articles_nums(self, nickname):
        """
        获取公众号的总共发布的文章数量

        Parameters
        ----------
        nickname : str
            需要爬取公众号名称

        Returns
        -------
        int
            文章总数
        """
        self.__verify_str(nickname, "nickname")
        try:
            return self.__get_articles_data(nickname, begin="0")["app_msg_cnt"]
        except Exception:
            raise Exception(u"公众号名称错误或cookie、token错误，请重新输入")

    def get_urls(self, nickname, begin=0, count=5):
        """
        获取公众号的每页的文章信息

        Parameters
        ----------
        nickname : str
            需要爬取公众号名称
        begin: str or int
            起始爬取的页数
        count: str or int
            每次爬取的数量，1-5

        Returns
        -------
        list:
            由每个文章信息构成的数组::

                [
                {
                    'aid': '2650949647_1',
                    'appmsgid': 2650949647,
                    'cover': 封面的url'digest': 文章摘要,
                    'itemidx': 1,
                    'link': 文章的url,
                    'title': 文章标题,
                    'update_time': 更新文章的时间戳
                },
                ]
            如果list为空则说明没有相关文章
        """
        self.__verify_str(nickname, "nickname")
        try:
            return self.__get_articles_data(
                nickname, begin=str(begin), count=str(count)
            )["app_msg_list"]
        except Exception:
            raise Exception(u"公众号名称错误或cookie、token错误，请重新输入")

    def latest_articles(self, biz):
        """
        获取公众号的最新页的文章信息

        Parameters
        ----------
        biz : str
            公众号的biz

        Returns
        -------
        list:
            由每个文章信息构成的数组::

                [
                {
                    'aid': '2650949647_1',
                    'appmsgid': 2650949647,
                    'cover': 封面的url'digest': 文章摘要,
                    'itemidx': 1,
                    'link': 文章的url,
                    'title': 文章标题,
                    'update_time': 更新文章的时间戳
                },
                ]
            如果list为空则说明没有相关文章
        """
        try:
            return self.__get_articles_data("", begin="0", biz=biz)["app_msg_list"]
        except Exception:
            raise Exception(u"公众号名称错误或cookie、token错误，请重新输入")

    def __get_articles_data(
        self,
        nickname,
        begin,
        biz=None,
        count=5,
        type_="9",
        action="list_ex",
        query=None,
    ):
        """
        Parameters
        ----------
        nickname : str
            需要爬取公众号名称
        begin: str or int
            起始爬取的页数
        biz : str
            公众号的biz
        count: str or int
            每次爬取的数量，1-5
        type_: str or int
            获取数据的方式，暂不知道具体用途
        action: str
            请求之后的行为动作，"list_ex"获取文章信息的json
        Returns
        -------
        json:
            文章信息的json::

                {
                'app_msg_cnt': 公众号发文章总数,
                'app_msg_list': 　一个数组(参看get_articles函数),
                'base_resp': {
                    'err_msg': 'ok',
                    'ret': 0
                }
                }
        """
        # 获取文章信息的url
        appmsg_url = "https://mp.weixin.qq.com/cgi-bin/appmsg"

        try:
            if nickname != "":
                # 获取公众号的fakeid
                official_info = self.official_info(nickname)
                self.params["fakeid"] = official_info[0]["fakeid"]
            elif biz != None:
                self.params["fakeid"] = biz
            else:
                raise Exception(u"请输入biz或者nickname")
        except Exception:
            raise Exception(u"公众号名称错误或cookie、token错误，请重新输入")

        # 增加/更改请求参数
        params = {
            "query": query if query != None else "",
            "begin": str(begin),
            "count": str(count),
            "type": str(type_),
            "action": action,
        }
        self.params.update(params)

        data = self.s.get(
            appmsg_url, headers=self.headers, params=self.params, proxies=self.proxies
        )
        return data.json()


class PC(object):
    """通过PC端的微信，获取需要爬取的微信公众号的推文链接"""

    def __init__(self, biz, uin, cookie, proxies={"http": None, "https": None}):
        """
        Parameters
        ----------
        __biz: str
            需要爬取公众号的id
        uin: str
            用户id
        cookies : str
            登录微信后获取的cookie

        """
        self.s = requests.session()
        self.__biz = biz
        self.uin = uin
        self.headers = {"Cookies": cookie}
        self.proxies = proxies

    def get_urls(self, key, offset="0"):
        """
        Parameters
        ----------
        key: str
            个人微信号登陆后获取的key
        offset: str or int
            获取起始的页数，从0开始，每次递增10（可以大于10，但是不好确认参数，所以递增10，之后再去重）
        Returns
        ----------
        list:
        由每个文章信息构成的数组，主要获取的参数`item['app_msg_ext_info']['content_url']`, `item['app_msg_ext_info']['title']`, `item['comm_msg_info']['datetime']`::

            import html
            消除转义 html.unescape(html.unescape(url)); eval(repr(url).replace('\\', ''))
            [
                {
                    'app_msg_ext_info': {
                        'audio_fileid': 0,
                        'author': '',
                        'content': '',
                        'content_url': 文章url，存在转义符'/'需要去除,
                        'copyright_stat': 100,
                        'cover': 文章封面url，存在转义符'/'需要去除,
                        'del_flag': 1,
                        'digest': '',
                        'duration': 0,
                        'fileid': 0,
                        'is_multi': 0,
                        'item_show_type': 8,
                        'malicious_content_type': 0,
                        'malicious_title_reason_id': 0,
                        'multi_app_msg_item_list': [],
                        'play_url': '',
                        'source_url': '',
                        'subtype': 9,
                        'title': 文章标题
                    },
                    'comm_msg_info': {
                        'content': '',
                        'datetime': 1536930840,
                        'fakeid': '2394588245',
                        'id': 1000000262,
                        'status': 2,
                        'type': 49
                    }
                }
            ]
        """
        self.params = {
            "action": "getmsg",
            "__biz": self.__biz,
            "f": "json",
            "offset": str(offset),
            "count": "10",
            "uin": self.uin,
            "key": key,
        }
        origin_url = "https://mp.weixin.qq.com/mp/profile_ext"

        msg_json = self.s.get(
            origin_url, params=self.params, headers=self.headers, proxies=self.proxies
        ).json()
        if "general_msg_list" in msg_json.keys():
            lst = [
                item
                for item in eval(msg_json["general_msg_list"])["list"]
                if "app_msg_ext_info" in item.keys()
            ]
            return lst

        raise Exception(
            "Failure:\n1.params is error, please check your params\n2.key is lose efficacy, please update your key"
        )


class Mobile(object):
    """通过移动端的wechat，获取需要爬取的微信公众号的推文链接"""

    def __init__(self, biz, cookie):
        """
        Parameters
        ----------
        __biz: str
            需要爬取公众号的id
        cookie : str
            登录微信后获取的cookie
        """
        self.s = requests.session()
        self.__biz = biz
        self.headers = {
            "User-Agent": "Mozilla/5.0 AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0Chrome/57.0.2987.132 MQQBrowser/6.2 Mobile",
            "Cookie": cookie,
        }

    def get_urls(self, appmsg_token, offset="0"):
        """
        Parameters
        ----------
        appmsg_token: str
            个人微信号登陆后获取的token
        offset: str or int
            获取起始的页数，从0开始，每次递增10（可以大于10，但是不好确认参数，所以递增10，之后再去重）
        
        Returns
        ----------
        list:
        由每个文章信息构成的数组::

            [
                {
                    'app_msg_ext_info': {
                        'audio_fileid': 0,
                        'author': '',
                        'content': '',
                        'content_url': 文章url，存在转义符'/'需要去除,
                        'copyright_stat': 100,
                        'cover': 文章封面url，存在转义符'/'需要去除,
                        'del_flag': 1,
                        'digest': '',
                        'duration': 0,
                        'fileid': 0,
                        'is_multi': 0,
                        'item_show_type': 8,
                        'malicious_content_type': 0,
                        'malicious_title_reason_id': 0,
                        'multi_app_msg_item_list': [],
                        'play_url': '',
                        'source_url': '',
                        'subtype': 9,
                        'title': 文章标题
                    },
                    'comm_msg_info': {
                        'content': '',
                        'datetime': 1536930840,
                        'fakeid': '2394588245',
                        'id': 1000000262,
                        'status': 2,
                        'type': 49
                    }
                }
            ]
        """
        self.params = {
            "action": "getmsg",
            "__biz": self.__biz,
            "f": "json",
            "offset": str(offset),
            "count": "10",
            "appmsg_token": appmsg_token,
        }
        origin_url = "https://mp.weixin.qq.com/mp/profile_ext"

        msg_json = self.s.get(
            origin_url, params=self.params, headers=self.headers, proxies=self.proxies
        ).json()
        if "general_msg_list" in msg_json.keys():
            lst = [
                item
                for item in eval(msg_json["general_msg_list"])["list"]
                if "app_msg_ext_info" in item.keys()
            ]
            return lst

        raise Exception(
            "Failure:\n1.params is error, please check your params\n2.key is lose efficacy, please update your key"
        )


class WeBook(object):
    """
    通过微信读书，获取需要爬取的微信公众号的推文链接
    vid是固定的微信账号
    skey是变动的
    需要注意抓取时间，如每次抓取暂停50s，则可以抓取51次左右。
    该接口也不会封禁（暂时来看），如果被判定频繁，只需要在移动端进行滑动验证码，则可以继续抓取
    """

    def __init__(
        self, skey, vid, user_agent=None, proxies={"http": None, "https": None}
    ):
        self.s = requests.session()
        self.base_url = "https://i.weread.qq.com/book/articles?bookId=MP_WXS_{}&count=20&offset={}&synckey={}"
        user_agent = (
            "WeRead/5.3.4 (iPhone; iOS 14.1; Scale/2.00)"
            if user_agent == None
            else user_agent
        )
        self.headers = {
            "User-Agent": user_agent,
            "Cookies": "wr_logined=1",
            "skey": skey,
            "vid": vid,
        }
        self.proxies = proxies

    def get_urls(self, bookid, offset="0"):
        url = self.base_url.format(bookid, offset, str(time.time()).split(".")[0])
        res = self.s.get(url, headers=self.headers, proxies=self.proxies)
        if "reviews" in res.json():
            item_lst = res.json()["reviews"]
            return [item["review"] for item in item_lst]
        else:
            print(res.json())
            return []



# coding: utf-8
import os
import re
import time

import requests
from bs4 import BeautifulSoup as bs

from .ArticlesUrls import PublicAccountsWeb


class AccountBiz(object):
    """通过公众号名称获取biz

    微信公众号网页版、清博、西瓜 ['xigu', 'qingbo', 'office']

    实测西瓜一次性可获取较多（目前西瓜的已废弃）


    """

    def __init__(
        self,
        cookie,
        token=None,
        method=None,
        t=120,
        proxies={
            "http": None,
            "https": None,
        },
    ):
        """
        Parameters
        ----------
        t: int
            获取一次时间间隔。西瓜10s, 清博10s, 公众号120s
        cookie: str
            平台登录的cookie
        token: str
            官方获取时需要token
        method: str
            三种获取方式，三选一
        """
        self.headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/65.0.3325.181 Safari/537.36",
            "Cookie": cookie,
        }
        assert method in ["xigua", "qingbo", "office"]
        self.method = method
        self.cookie = cookie
        self.token = token
        self.t = t
        self.biz_name = "{}, {}"
        self.proxies = proxies

    def run(self, nickname_lst):
        if self.method == "xigua":
            return self.xigua(nickname_lst)
        elif self.method == "qingbo":
            return self.qingbo(nickname_lst)
        else:
            s = PublicAccountsWeb(cookie=self.cookie, token=self.token, proxies=self.proxies)
            return self.office(s, nickname_lst)

    def office(self, s, nickname_lst):
        self.res_lst = []
        for nickname in nickname_lst:
            try:
                official_infos = s.official_info(nickname)
                if official_infos:
                    official_info = official_infos[0]
                    biz = official_info["fakeid"]
                    tmp = self.biz_name.format(biz, official_info["nickname"])
                    self.res_lst.append(tmp)
                time.sleep(self.t)
            except Exception as e:
                print(e)
                return self.res_lst
        return self.res_lst

    def xigua(self, nickname_lst):
        url = "https://data.xiguaji.com/Search/SearchAct/?type=1&key={}"
        self.res_lst = []
        for nickname in nickname_lst:
            try:
                s = requests.get(
                    url.format(nickname), headers=self.headers, proxies=self.proxies
                )
                soup = bs(s.text, "lxml")
                infos = soup.find_all(class_="number-details")
                if infos:
                    info = infos[0]
                    nickname = info.h3.text.strip()
                    try:
                        biz = info.img["src"].split("__biz=")[1].split("&")[0]
                    except Exception as e:
                        continue
                        print(e)
                    self.res_lst.append(self.biz_name.format(biz, nickname))
                time.sleep(self.t)
            except Exception as e:
                print(e)
                return self.res_lst
        return self.res_lst

    def qingbo(self, nickname_lst):
        url = "http://www.gsdata.cn/query/wx?q={}"
        self.res_lst = []
        for nickname in nickname_lst:
            try:
                s = requests.get(
                    url.format(nickname), headers=self.headers, proxies=self.proxies
                )
                biz_lst = re.findall(
                    r'<input type="hidden" class="biz" value="(.+)">?', s.text
                )
                if biz_lst != []:
                    nicknames_lst = re.findall(
                        r'<span class="color-pink">(.+)</span>', s.text
                    )
                    tmp = self.biz_name.format(biz_lst[0], nicknames_lst[0])
                    self.res_lst.append(tmp)
                time.sleep(self.t)
            except Exception as e:
                print(e)
                return self.res_lst
        return self.res_lst


if __name__ == "__main__":
    nickname_lst = ["科技美学", "AppSo", "InfoQ"]
    cookie = ""
    ab = AccountBiz(cookie, method="xigua", t=10)
    res_lst = ab.run(nickname_lst)

    fname = "1.txt"
    with open(fname, "w", encoding="utf-8") as f:
        f.write("\n".join(item for item in res_lst))



# coding:  utf-8
import re

import requests
from bs4 import BeautifulSoup as bs


class ArticlesInfo(object):
    """登录WeChat，获取更加详细的推文信息。如点赞数、阅读数、评论等"""

    def __init__(self, appmsg_token, cookie, proxies={"http": None, "https": None}):
        """
        初始化参数

        Parameters
        ----------
        cookie: str
            点开微信公众号文章抓包工具获取的cookie
        appmsg_token: str
            点开微信公众号文章抓包工具获取的appmsg_token
        """
        self.s = requests.session()
        self.s.trust_env = False
        self.appmsg_token = appmsg_token
        self.headers = {
            "User-Agent": "Mozilla/5.0 AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0Chrome/57.0.2987.132 MQQBrowser/6.2 Mobile",
            "Cookie": cookie,
        }
        self.data = {
            "is_only_read": "1",
            "is_temp_url": "0",
            "appmsg_type": "9",  # 新参数，不加入无法获取like_num
        }
        self.proxies = proxies
        self.too_frequently_text = "你的访问过于频繁，需要从微信打开验证身份，是否需要继续访问当前页面"

    def __verify_url(self, article_url):
        """
        简单验证文章url是否符合要求

        Parameters
        ----------
        article_url: str
            文章链接
        """
        verify_lst = ["mp.weixin.qq.com", "__biz", "mid", "sn", "idx"]
        for string in verify_lst:
            if string not in article_url:
                raise Exception("params is error, please check your article_url")

    def read_like_nums(self, article_url):
        """
        获取阅读数和点赞数

        Parameters
        ----------
        article_url: str
            文章链接

        Returns
        -------
        (int, int):
            阅读数、点赞数
        """
        try:
            appmsgstat = self.__get_appmsgext(article_url)["appmsgstat"]
            return (
                appmsgstat["read_num"],
                appmsgstat["like_num"],
                appmsgstat["old_like_num"],
            )
        except Exception:
            raise Exception("params is error, please check your article_url")

    def comments(self, article_url):
        """
        获取文章评论

        Parameters
        ----------
        article_url: str
            文章链接

        Returns
        -------
        json::

            {
                "base_resp": {
                    "errmsg": "ok",
                    "ret": 0
                },
                "elected_comment": [
                    {
                        "content": 用户评论文字,
                        "content_id": "6846263421277569047",
                        "create_time": 1520098511,
                        "id": 3,
                        "is_from_friend": 0,
                        "is_from_me": 0,
                        "is_top": 0, 是否被置顶
                        "like_id": 10001,
                        "like_num": 3,
                        "like_status": 0,
                        "logo_url": "http://wx.qlogo.cn/mmhead/OibRNdtlJdkFLMHYLMR92Lvq0PicDpJpbnaicP3Z6kVcCicLPVjCWbAA9w/132",
                        "my_id": 23,
                        "nick_name": 评论用户的名字,
                        "reply": {
                            "reply_list": [ ]
                        }
                    }
                ],
                "elected_comment_total_cnt": 3, 评论总数
                "enabled": 1,
                "friend_comment": [ ],
                "is_fans": 1,
                "logo_url": "http://wx.qlogo.cn/mmhead/Q3auHgzwzM6GAic0FAHOu9Gtv5lEu5kUqO6y6EjEFjAhuhUNIS7Y2AQ/132",
                "my_comment": [ ],
                "nick_name": 当前用户名,
                "only_fans_can_comment": false
            }
        """
        __biz, _, idx, _ = self.__get_params(article_url)
        getcomment_url = "https://mp.weixin.qq.com/mp/appmsg_comment?action=getcomment&__biz={}&idx={}&comment_id={}&limit=100"
        try:
            comment_id = self.__get_comment_id(article_url)
            if comment_id == "":
                return {}
            url = getcomment_url.format(__biz, idx, comment_id)
            return self.s.get(url, headers=self.headers, proxies=self.proxies).json()
        except Exception as e:
            print(e)
            return {}

    def __get_comment_id(self, article_url):
        """
        获取comment_id

        Parameters
        ----------
        article_url: str
            文章链接

        Returns
        -------
        str:
            comment_id获取评论必要参数
        """
        res = self.s.get(article_url, data=self.data, proxies=self.proxies)
        # 使用正则提取comment_id
        comment_id = re.findall(r'comment_id = "(\d+)"', res.text)
        # 增加其他情况，感谢@[harry7756](https://github.com/harry7756)建议
        if len(comment_id) == 0:
            comment_id = re.findall(r"(?<=comment_id.DATA\'\)\s\:\s\')[0-9]+", res.text)
        if len(comment_id) > 0:
            return comment_id[0]
        return ""

    def __get_params(self, article_url):
        """
        解析文章url, 获取必要的请求参数

        Parameters
        ----------
        article_url: str
            文章链接

        Returns
        -------
        (str, str, str, str):
            __biz, mid, idx, sn
        """
        # 简单验证文章的url是否正确
        self.__verify_url(article_url)
        # 切分url, 提取相应的参数
        string_lst = article_url.split("?")[1].split("&")
        dict_value = [string[string.index("=") + 1 :] for string in string_lst]
        __biz, mid, idx, sn, *_ = dict_value
        sn = sn[:-3] if sn[-3] == "#" else sn

        return __biz, mid, idx, sn

    def __get_appmsgext(self, article_url):
        """
        获取每篇文章具体信息

        Parameters
        ----------
        article_url: str
            文章链接

        Returns
        -------
        json:
            文章具体信息的json::

                {
                    'advertisement_info': [],
                    'advertisement_num': 0,
                    'appmsgstat': {'is_login': True,
                    'like_num': 12,
                    'liked': False,
                    'read_num': 288,
                    'real_read_num': 0,
                    'ret': 0,
                    'show': True},
                    'base_resp': {'wxtoken': 2045685972},
                    'reward_head_imgs': []
                }
        """
        __biz, mid, idx, sn = self.__get_params(article_url)

        # 将params参数换到data中请求。这一步貌似不换也行
        origin_url = "https://mp.weixin.qq.com/mp/getappmsgext?"
        appmsgext_url = origin_url + "appmsg_token={}&x5=0".format(self.appmsg_token)
        self.data["__biz"] = __biz
        self.data["mid"] = mid
        self.data["sn"] = sn
        self.data["idx"] = idx

        # appmsgext_url = origin_url + "__biz={}&mid={}&sn={}&idx={}&appmsg_token={}&x5=1".format(
        #     __biz, mid, sn, idx, self.appmsg_token)
        appmsgext_json = requests.post(
            appmsgext_url, headers=self.headers, data=self.data, proxies=self.proxies
        ).json()

        if "appmsgstat" not in appmsgext_json.keys():
            raise Exception("get info error, please check your cookie and appmsg_token")
        return appmsgext_json

    def __get_content(self, url):
        return self.s.get(url.strip(), headers=self.headers, proxies=self.proxies).text

    def content(self, url, html_text=None):
        if html_text == None:
            html_text = self.__get_content(url)

        soup = bs(html_text, "lxml")
        if self.too_frequently_text in html_text:
            raise SystemError("访问频繁！")
        # js加载
        # html.text.split('var content = ')[1].split('var')[0].strip()
        # soup.find(id="js_panel_like_title").text
        try:
            body = soup.find(class_="rich_media_area_primary_inner")
            content_p = body.find(class_="rich_media_content")
            if content_p:
                imgs = body.find_all("img")
                return content_p.text.strip(), len(content_p.text.strip()), len(imgs)
            else:
                content_p = soup.find(id="js_panel_like_title").text.strip()
                return content_p, len(content_p), 0
        except:
            return "", 0, 0

    def complete_content(self, url, html_text=None):
        if html_text == None:
            html_text = self.__get_content(url)

        innerlink_flag = 0
        video_flag = 0
        imgs_flag = 0
        account_key_string_lst = [
            "此帐号已被屏蔽, 内容无法查看",
            "该公众号已迁移",
            "此帐号已自主注销，内容无法查看",
            "此帐号处于帐号迁移流程中",
            "此帐号被投诉且经审核涉嫌侵权。此帐号已注销，内容无法查看。",
        ]
        key_string_lst = [
            "该内容已被发布者删除",
            "此内容因违规无法查看",
            "此内容被投诉且经审核涉嫌侵权，无法查看。",
            "此内容被多人投诉，相关的内容无法进行查看。",
        ]

        if self.too_frequently_text in html_text:
            raise SystemError("访问频繁！")

        for key_string in key_string_lst:
            if key_string in html_text:
                title = key_string
                return (
                    "",
                    title,
                    "",
                    "",
                    imgs_flag,
                    innerlink_flag,
                    video_flag,
                )
        for key_string in account_key_string_lst:
            if key_string in html_text:
                title = key_string
                return (
                    "",
                    title,
                    "",
                    "",
                    imgs_flag,
                    innerlink_flag,
                    video_flag,
                )

        try:
            title = html_text.split("<h2")[1].split("</h2")[0].split(">")[1].strip()
        except Exception as e:
            title = ""

        if 'ct = "' in html_text:
            timestamp = int(html_text.split('ct = "')[1].split('";')[0].strip())
        else:
            timestamp = (
                html_text.split("ct = ")[1].split("|| '")[1].split("'")[0].strip()
            )
            timestamp = int(timestamp) if timestamp != "" else 0

        if "copyright" in html_text:
            if '_copyright_stat = "' in html_text:
                copyright_stat = int(
                    html_text.split('_copyright_stat = "')[1].split('";')[0].strip()
                )
            else:
                copyright_stat = (
                    html_text.split("copyright_stat =")[1]
                    .split("|| '")[1]
                    .split("'")[0]
                    .strip()
                )
                copyright_stat = int(copyright_stat) if copyright_stat != "" else 0
        else:
            copyright_stat = 0

        if "nick_name = " in html_text:
            nickname = (
                html_text.split("nick_name = ")[1]
                .split("|| '")[1]
                .split("'")[0]
                .strip()
            )
        else:
            nickname = html_text.split('nickname = "')[1].split('";')[0].strip()

        soup = bs(html_text, "lxml")

        body = soup.find(class_="rich_media_area_primary_inner")
        imgs_flag = len(body.find_all("img"))
        a_lst = body.find_all("a")
        for a_item in a_lst:
            if "tab" in a_item.attrs.keys() and a_item.attrs["tab"] == "innerlink":
                innerlink_flag = 1
                break
        video_lst = body.find_all(class_="js_video_channel_container")
        if len(video_lst) > 0:
            video_flag = 1
        return (
            nickname,
            title,
            timestamp,
            copyright_stat,
            imgs_flag,
            innerlink_flag,
            video_flag,
        )



# coding: utf-8
import os
from pprint import pprint
from wechatarticles import PublicAccountsWeb
from wechatarticles.utils import save_json

if __name__ == "__main__":
    # 模拟登录微信公众号平台，获取微信文章的url
    cookie = "cookie"
    token = "token"
    nickname = "nickname"

    paw = PublicAccountsWeb(cookie=cookie, token=token)
    # articles_sum = paw.articles_nums(nickname)
    article_data = paw.get_urls(nickname, begin="0", count="5")
    # official_info = paw.official_info(nickname)

    # print("articles_sum:", end=" ")
    # print(articles_sum)
    print("artcles_data:")
    pprint(article_data)
    # print("official_info:")
    # pprint(official_info)

    save_json("paw.json", article_data)



# coding:utf-8
import sys

from wechatarticles.proxy import ReqIntercept, RspIntercept, MitmProxy, ProxyHandle
import re
import urllib


"""
证书生成note
openssl req -newkey rsa:2048 -nodes -keyout CA_private.pem -x509 -subj "/CN=MY_CA" -days 365 -out CA_cert.crt
openssl genrsa -out client.pem 1024
openssl x509  -req -days 3650 -in client.csr -CA CA_cert.crt -CAkey CA_private.pem  -CAcreateserial -out client.crt
openssl genrsa -out server.pem 1024
openssl req -new -key server.pem -out server.csr -subj "/CN=server"
openssl x509  -req -days 3650 -in server.csr -CA CA_cert.crt -CAkey CA_private.pem  -CAcreateserial -out server.crt

证书使用的是原项目的，若对安全性要求高，建议重新生成
"""


class MyProxyHandle(ProxyHandle):
    def hook_init(self):
        self.filter_url_lst = [
            "mp.weixin.qq.com/mp",
            "mp.weixin.qq.com/s",
            "weread.qq.com/",
        ]


def get_request(response):
    data = response.request
    if "Cookie" in data.headers.keys():
        cookies = data.headers["Cookie"]
    elif "cookie" in data.headers.keys():
        cookies = data.headers["cookie"]
    else:
        cookies = ""
    return cookies, data.url, response.get_text()


def parse_params(url):
    nickname = re.findall("__biz=(.+?)&", url)[0]  # 此处仅作占位标识，同biz表示
    biz = re.findall("__biz=(.+?)&", url)[0]
    key = re.findall("key=(.+?)&", url)[0]
    uin = re.findall("uin=(.+?)&", url)[0]
    uin = urllib.parse.unquote(uin)
    nickname = urllib.parse.unquote(nickname)
    biz = urllib.parse.unquote(biz)
    return biz, nickname, uin, key


class DebugInterceptor(ReqIntercept, RspIntercept):
    def deal_request(self, request):
        return request

    def deal_response(self, response):
        url = response.request.url
        
        # 历史文章链接
        if "mp.weixin.qq.com/mp/profile_ext" in url and "action=home" in url:
            cookies, url, text = get_request(response)
            with open("history_url.txt", "w", encoding="utf-8") as f:
                f.write(
                    text.split("var msgList = '")[1]
                    .split("';")[0]
                    .replace("&quot;", '"')
                    .replace("amp;", "")
                )
            nickname = text.split('nickname = "')[1].split('".html(false) || "";')[
                0
            ]
            biz, _, uin, key = parse_params(url)


        if "mp.weixin.qq.com/mp/getappmsgext" in url and "biz" in url:
            cookies, url, _ = get_request(response)
            appmsg_token = re.findall("appmsg_token=(.+?)&", url)[0]
            print("cookie = '{}'".format(cookies))
            print("appmsg_token = r'{}'".format(appmsg_token))
            with open("params/cookie_token.txt", "w", encoding="utf-8") as f:
                f.write(cookies + "\n")
                f.write(appmsg_token)

        # 评论
        if "mp.weixin.qq.com/mp/appmsg_comment" in url and "biz" in url:
            _, _, text = get_request(response)
            with open("comment.txt", "w", encoding="utf-8") as f:
                f.write(text)

        return response


def run(port=8080):
    baseproxy = MitmProxy(
        server_addr=("", port),
        RequestHandlerClass=MyProxyHandle,
        bind_and_activate=True,
        https=True,
        ca_file="ca.pem",
        cert_file="ca.crt",
    )
    baseproxy.register(DebugInterceptor)
    baseproxy.serve_forever()


if __name__ == "__main__":
    run()


# coding: utf-8

from .AccountBiz import AccountBiz
from .ArticlesAPI import ArticlesAPI
from .ArticlesInfo import ArticlesInfo
from .ArticlesUrls import PC, Mobile, PublicAccountsWeb, WeBook
from .Url2Html import Url2Html



# coding: utf-8
import json
import os
import random
import time
from pprint import pprint

import pandas as pd
from wechatarticles import ArticlesInfo
from wechatarticles.utils import get_history_urls, verify_url

# 快速获取大量文章urls（利用历史文章获取链接）


def save_xlsx(fj, lst):
    df = pd.DataFrame(lst, columns=["url", "title", "date", "read_num", "like_num"])
    df.to_excel(fj + ".xlsx", encoding="utf-8")


def demo(lst):
    # 抓取示例，供参考，不保证有效
    fj = "公众号名称"
    item_lst = []
    for i, line in enumerate(lst, 0):
        print("index:", i)
        # item = json.loads('{' + line + '}', strict=False)
        item = line
        timestamp = item["comm_msg_info"]["datetime"]
        ymd = time.localtime(timestamp)
        date = "{}-{}-{}".format(ymd.tm_year, ymd.tm_mon, ymd.tm_mday)

        infos = item["app_msg_ext_info"]
        url_title_lst = [[infos["content_url"], infos["title"]]]
        if "multi_app_msg_item_list" in infos.keys():
            url_title_lst += [
                [info["content_url"], info["title"]]
                for info in infos["multi_app_msg_item_list"]
            ]

        for url, title in url_title_lst:
            try:
                if not verify_url(url):
                    continue
                # 获取文章阅读数在看点赞数
                read_num, like_num, old_like_num = ai.read_like_nums(url)
                print(read_num, like_num)
                item_lst.append([url, title, date, read_num, like_num])
                time.sleep(random.randint(5, 10))
            except Exception as e:
                print(e)
                flag = 1
                break
            finally:
                save_xlsx(fj, item_lst)

        if flag == 1:
            break

    save_xlsx(fj, item_lst)


if __name__ == "__main__":
    # 需要抓取公众号的__biz参数
    biz = ""
    # 个人微信号登陆后获取的uin
    uin = ""
    # 个人微信号登陆后获取的key，隔段时间更新
    key = ""

    lst = get_history_urls(
        biz, uin, key, lst=[], start_timestamp=0, start_count=0, end_count=10
    )
    print("抓取到的文章链接")
    print(lst)

    # 个人微信号登陆后获取的token
    appmsg_token = ""
    # 个人微信号登陆后获取的cookie
    cookie = ""
    # 获取点赞数、阅读数、评论信息
    ai = ArticlesInfo(appmsg_token, cookie)

    # url：微信文章链接. lst[0]["app_msg_ext_info"]["content_url"]
    read_num, like_num, old_like_num = ai.read_like_nums(url)
    item = ai.comments(url)
    print("阅读：{}; 在看: {}; 点赞: {}".format(read_num, like_num, old_like_num))
    print("评论信息")
    pprint(item)



# coding: utf-8
import os
import re
import time

import requests
from bs4 import BeautifulSoup as bs


class Url2Html(object):
    """根据微信文章链接下载为本地HTML文件"""

    def __init__(self, img_path=None):
        """
        Parameters
        ----------
        img_path: str
            本地存储图片的路径，采用绝对路径的方式引用图片。可不下载图片
        """
        self.data_src_re = re.compile(r'data-src="(.*?)"')
        self.data_croporisrc_re = re.compile(r'data-croporisrc="(.*?)"')
        self.src_re = re.compile(r'src="(.*?)"')
        self.img_path = img_path

    @staticmethod
    def replace_name(title):
        """
        对进行标题替换，确保标题符合windows的命名规则

        Parameters
        ----------
        title: str
            文章标题

        Returns
        ----------
        str: 替换后的文章标题
        """
        rstr = r"[\/\\\:\*\?\"\<\>\|]"  # '/ \ : * ? " < > |'
        title = re.sub(rstr, "", title).replace("|", "").replace("\n", "")
        return title

    def download_img(self, url):
        """
        Parameters
        ----------
        url: str
            图片链接

        Returns
        ----------
        str: 下载图片的本地路径
        """
        # 根据链接提取图片名
        name = "{}.{}".format(url.split("/")[-2], url.split("/")[3].split("_")[-1])
        imgpath = os.path.join(self.img_path, name)
        # 如果该图片已被下载，可以无需再下载，直接返回路径即可
        if os.path.isfile(imgpath):
            with open(imgpath, "rb") as f:
                img = f.read()
            return imgpath, img

        response = requests.get(url, proxies=self.proxies)
        img = response.content
        with open(imgpath, "wb") as f:
            f.write(img)
        return imgpath, img

    def replace_img(self, html):
        """
        根据提供的html源码找出其中的图片链接，并对其进行替换

        Parameters
        ----------
        html: str
            文章HTML源码

        Returns
        ----------
        str: 替换html中在线图片链接为本地图片路径
        """
        data_croporisrc_lst = self.data_croporisrc_re.findall(html)
        data_src_lst = self.data_src_re.findall(html)
        src_lst = self.src_re.findall(html)

        img_url_lst = data_croporisrc_lst + data_src_lst + src_lst
        img_lst = []
        for img_url in img_url_lst:
            if "mmbiz.qpic.cn" in img_url:
                # print(img_url)
                data_src, img = self.download_img(img_url)
                img_lst.append([data_src, img])
                # 以绝对路径的方式替换图片
                html = html.replace(img_url, data_src).replace("data-src=", "src=")
        return html, img_lst

    @staticmethod
    def get_title(html):
        """
        根据提供的html源码提取文章中的标题

        Parameters
        ----------
        html: str
            文章HTML源码

        Returns
        ----------
        str: 根据HTML获取文章标题
        """
        try:
            # title = html.split('activity-name">')[1].split('</h2')[0].strip()
            title = html.split("<h2")[1].split("</h2")[0].split(">")[1].strip()
            return title
        except Exception as e:
            print(e)
            print(html.split("<h2")[1].split("</h2")[0])
            return ""

    @staticmethod
    def article_info(html):
        """
        根据提供的html源码提取文章中的公众号和作者

        Parameters
        ----------
        html: str
            文章HTML源码

        Returns
        ----------
        (str, str): 公众号名字和作者名字
        """
        # account = (
        #     html.split('rich_media_meta rich_media_meta_text">')[1]
        #     .split("</span")[0]
        #     .strip()
        # )
        account = html.split('nickname = "')[1].split('"')[0]
        author = html.split('id="js_name">')[1].split("</a")[0].strip()
        return account, author

    @staticmethod
    def get_timestamp(html):
        """
        根据提供的html源码提取文章发表的时间戳

        Parameters
        ----------
        html: str
            文章HTML源码

        Returns
        ----------
        int: 文章发表的时间戳
        """
        timestamp = int(html.split('ct = "')[1].split('";')[0].strip())
        return timestamp

    @staticmethod
    def timestamp2date(timestamp):
        """
        时间戳转日期

        Parameters
        ----------
        timestamp: int
                时间戳

        Returns
        ----------
        str: 文章发表的日期，yyyy-mm-dd
        """
        ymd = time.localtime(timestamp)
        date = "{}-{}-{}".format(ymd.tm_year, ymd.tm_mon, ymd.tm_mday)
        return date

    def rename_title(self, title, html):
        # 自动获取文章标题
        if title == None:
            title = self.get_title(html)
        if title == "":
            return ""
        title = self.replace_name(title)

        if self.account == None:
            try:
                account_name = self.article_info(html)[0]
            except:
                account_name = "未分类"
        else:
            account_name = self.account
        try:
            date = self.timestamp2date(self.get_timestamp(html))
        except:
            date = ""
        # try:
        if not os.path.isdir(account_name):
            os.mkdir(account_name)
        # except:
        #     account_name = '未分类'
        #     if not os.path.isdir(account_name):
        #         os.mkdir(account_name)

        title = os.path.join(
            account_name, "[{}]-{}-{}".format(account_name, date, title)
        )
        return title

    @staticmethod
    def download_media(html, title):
        soup = bs(html, "lxml")
        # mp3
        mpvoice_item_lst = soup.find_all("mpvoice")
        base_url = "https://res.wx.qq.com/voice/getvoice?mediaid="
        for i, item in enumerate(mpvoice_item_lst, 1):
            if os.path.isfile("{}-{}.mp3".format(title, i)):
                continue
            doc = requests.get(base_url + item["voice_encode_fileid"])
            with open("{}-{}.mp3".format(title, i), "wb") as f:
                f.write(doc.content)

        # video
        if os.path.isfile("{}.mp4".format(title)):
            return ""
        video_url = re.findall(r"url: \'(.+)\',\n", html)
        if video_url:
            video_url = [url for url in video_url if "videoplayer" not in url]
            if video_url:
                video_url = video_url[0].replace(r"\x26", "&")
                doc = requests.get(video_url)
                with open("{}.mp4".format(title), "wb") as f:
                    f.write(doc.content)

    @staticmethod
    def test_replace_img(html):
        return html.replace("data-src=", "src=").replace("wx_fmt=jpeg", "wx_fmt=web")

    def run(self, url, mode, proxies={"http": None, "https": None}, **kwargs):
        """
        Parameters
        ----------
        url: str
             微信文章链接
        mode: int
            运行模式
            1: 返回html源码，不下载图片
            2: 返回html源码，下载图片但不替换图片路径
            3: 返回html源码，下载图片且替换图片路径
            4: 保存html源码，下载图片且替换图片路径
            5: 保存html源码，下载图片且替换图片路径，并下载视频与音频
            6: 返回html源码，不下载图片，替换src和图片为web
        kwargs:
            account: 公众号名
            title: 文章名
            date: 日期
            proxies: 代理
            img_path: 图片下载路径

        Returns
        ----------
        str: HTML源码或消息
        """
        self.proxies = proxies
        if mode == 1:
            return requests.get(url, proxies=proxies).text
        elif mode == 6:
            return self.test_replace_img(requests.get(url, proxies=proxies).text)
        elif mode in [2, 3, 4, 5]:
            if "img_path" in kwargs.keys():
                self.img_path = kwargs["img_path"]
            else:
                return "{} 请输入保存图片路径!".format(url)
            if mode == 2:
                return requests.get(url, proxies=proxies).text
            elif mode == 3:
                html = requests.get(url, proxies=proxies).text
                html_img, _ = self.replace_img(html)
                return html_img
            else:
                account = kwargs["account"] if "account" in kwargs.keys() else None
                self.account = account
                title = kwargs["title"] if "title" in kwargs.keys() else None
                date = kwargs["date"] if "date" in kwargs.keys() else None
                proxies = kwargs["proxies"] if "proxies" in kwargs.keys() else None
                if self.account and title and date:
                    title = os.path.join(
                        self.account,
                        "[{}]-{}-{}".format(
                            self.account, date, self.replace_name(title)
                        ),
                    )
                    if os.path.isfile("{}.html".format(title)):
                        return 0
                    html = requests.get(url, proxies=proxies).text
                else:
                    html = requests.get(url, proxies=proxies).text
                    title = self.rename_title(title, html)

                if mode == 5:
                    try:
                        self.download_media(html, title)
                    except Exception as e:
                        print(e)
                        print(title)
                html_img, _ = self.replace_img(html)
                with open("{}.html".format(title), "w", encoding="utf-8") as f:
                    f.write(html_img)
                return "{} success!".format(url)
        else:
            print("please input correct mode num")
            return "failed!"


if __name__ == "__main__":

    url_lst = [
        "http://mp.weixin.qq.com/s?__biz=MTc5MTU3NTYyMQ==&mid=2650742058&idx=2&sn=1da6e9ddd1a0281e8c548fb30f8387f0&chksm=5afd0c006d8a851648e91e8cdaebfc2ab61d2518cbba369749b0a29cccb4781c622f966e6b04#rd"
    ]
    uh = Url2Html()
    for url in url_lst:
        s = uh.run(url, mode=4, img_path="D:\\imgs")
        print(s)



# coding: utf-8

from .ArticlesUrls import PublicAccountsWeb
from .ArticlesInfo import ArticlesInfo


class ArticlesAPI(object):
    """
    整合PublicAccountsWeb和ArticlesInfo，该API慎用，不再维护
    """

    def __init__(
        self,
        username=None,
        password=None,
        official_cookie=None,
        token=None,
        appmsg_token=None,
        wechat_cookie=None,
        outfile=None,
    ):
        """
        初始化参数

        Parameters
        ----------
        username: str
            用户账号
        password: str
            用户密码
        official_cookie : str
            登录微信公众号平台之后获取的cookie
        token : str
            登录微信公众号平台之后获取的token
        wechat_cookie: str
            点开微信公众号文章抓包工具获取的cookie
        appmsg_token: str
            点开微信公众号文章抓包工具获取的appmsg_token
        """
        # 两种登录方式, 扫码登录和手动输入登录
        if (token != None) and (official_cookie != None):
            self.official = PublicAccountsWeb(cookie=official_cookie, token=token)
        elif (username != None) and (password != None):
            self.official = PublicAccountsWeb(username=username, password=password)
        else:
            raise SystemError("please check your params")

        # 支持两种方式， mitmproxy自动获取参数和手动获取参数
        if (appmsg_token == None) and (wechat_cookie == None) and (outfile != None):
            raise SystemError("Not support this method")
        elif (appmsg_token != None) and (wechat_cookie != None):
            self.appmsg_token, self.cookie = appmsg_token, wechat_cookie
        else:
            raise SystemError("please check your params")

        self.wechat = ArticlesInfo(self.appmsg_token, self.cookie)

    def complete_info(self, nickname, begin=0, count=5):
        """
        获取公众号的抓取的文章文章信息

        Parameters
        ----------
        nickname: str
            公众号名称
        begin: str or int
            起始爬取的页数
        count: str or int
            每次爬取的数量，1-5

        Returns
        -------
        list:
            由每个文章信息构成的数组::

                [
                    {
                        'aid': '2650949647_1',
                        'appmsgid': 2650949647,
                        'comments': 文章评论信息
                            {
                                "base_resp": {
                                    "errmsg": "ok",
                                    "ret": 0
                                },
                                "elected_comment": [
                                    {
                                        "content": 用户评论文字,
                                        "content_id": "6846263421277569047",
                                        "create_time": 1520098511,
                                        "id": 3,
                                        "is_from_friend": 0,
                                        "is_from_me": 0,
                                        "is_top": 0, 是否被置顶
                                        "like_id": 10001,
                                        "like_num": 3,
                                        "like_status": 0,
                                        "logo_url": "http://wx.qlogo.cn/mmhead/OibRNdtlJdkFLMHYLMR92Lvq0PicDpJpbnaicP3Z6kVcCicLPVjCWbAA9w/132",
                                        "my_id": 23,
                                        "nick_name": 评论用户的名字,
                                        "reply": {
                                            "reply_list": [ ]
                                        }
                                    }
                                ],
                                "elected_comment_total_cnt": 3, 评论总数
                                "enabled": 1,
                                "friend_comment": [ ],
                                "is_fans": 1,
                                "logo_url": "http://wx.qlogo.cn/mmhead/Q3auHgzwzM6GAic0FAHOu9Gtv5lEu5kUqO6y6EjEFjAhuhUNIS7Y2AQ/132",
                                "my_comment": [ ],
                                "nick_name": 当前用户名,
                                "only_fans_can_comment": false
                            },
                        'cover': 封面的url'digest': 文章摘要,
                        'itemidx': 1,
                        'like_num': 18, 文章点赞数
                        'link': 文章的url,
                        'read_num': 610, 文章阅读数
                        'title': 文章标题,
                        'update_time': 更新文章的时间戳
                    },
                ]
        如果list为空则说明没有相关文章
        """
        # 获取文章数据
        article_data = self.official.get_urls(
            nickname, begin=str(begin), count=str(count)
        )

        # 提取每个文章的url，获取文章的点赞、阅读、评论信息，并加入到原来的json中
        for data in article_data:
            article_url = data["link"]
            comments = self.wechat.comments(article_url)
            read_like_nums = self.wechat.read_like_nums(article_url)
            data["comments"] = comments
            data["read_num"], data["like_num"], data["old_like_num"] = read_like_nums

        return article_data

    def __extract_info(self, articles_data):
        # 提取每个文章的url，获取文章的点赞、阅读、评论信息，并加入到原来的json中
        for data in articles_data:
            article_url = data["link"]
            comments = self.wechat.comments(article_url)
            read_like_nums = self.wechat.read_like_nums(article_url)
            data["comments"] = comments
            data["read_num"], data["like_num"], data["old_like_num"] = read_like_nums

        return articles_data

    def continue_info(self, nickname, begin=0):
        """
        自动获取公众号的抓取的文章文章信息，直到爬取失败为止

        Parameters
        ----------
        nickname: str
            公众号名称
        begin: str or int
            起始爬取的页数

        Returns
        -------
        list:
            由每个文章信息构成的数组::

                [
                {
                    'aid': '2650949647_1',
                    'appmsgid': 2650949647,
                    'comments': 文章评论信息
                        {
                            "base_resp": {
                                "errmsg": "ok",
                                "ret": 0
                            },
                            "elected_comment": [
                                {
                                    "content": 用户评论文字,
                                    "content_id": "6846263421277569047",
                                    "create_time": 1520098511,
                                    "id": 3,
                                    "is_from_friend": 0,
                                    "is_from_me": 0,
                                    "is_top": 0, 是否被置顶
                                    "like_id": 10001,
                                    "like_num": 3,
                                    "like_status": 0,
                                    "logo_url": "http://wx.qlogo.cn/mmhead/OibRNdtlJdkFLMHYLMR92Lvq0PicDpJpbnaicP3Z6kVcCicLPVjCWbAA9w/132",
                                    "my_id": 23,
                                    "nick_name": 评论用户的名字,
                                    "reply": {
                                        "reply_list": [ ]
                                    }
                                }
                            ],
                            "elected_comment_total_cnt": 3, 评论总数
                            "enabled": 1,
                            "friend_comment": [ ],
                            "is_fans": 1,
                            "logo_url": "http://wx.qlogo.cn/mmhead/Q3auHgzwzM6GAic0FAHOu9Gtv5lEu5kUqO6y6EjEFjAhuhUNIS7Y2AQ/132",
                            "my_comment": [ ],
                            "nick_name": 当前用户名,
                            "only_fans_can_comment": false
                        },
                    'cover': 封面的url'digest': 文章摘要,
                    'itemidx': 1,
                    'like_num': 18, 文章点赞数
                    'link': 文章的url,
                    'read_num': 610, 文章阅读数
                    'title': 文章标题,
                    'update_time': 更新文章的时间戳
                },
                ]
        如果list为空则说明没有相关文章
        """
        article_datas = []
        count = 5
        while True:
            try:
                # 获取文章数据
                article_datas.append(
                    self.official.get_urls(nickname, begin=str(begin), count=str(count))
                )
            except Exception as e:
                print(e)
                break
            begin += count
            if begin > 40:
                break

        def flatten(x):
            return [y for l in x for y in flatten(l)] if type(x) is list else [x]

        # flatten = lambda x: [y for l in x for y in flatten(l)] if type(x) is list else [x]
        print("第{}篇文章爬取失败，请过段时间再次尝试或换个帐号继续爬取".format(begin))
        return self.__extract_info(flatten(article_datas))



# coding: utf-8
import os
from pprint import pprint
from wechatarticles import ArticlesInfo

if __name__ == "__main__":
    # 登录微信PC端获取文章信息
    appmsg_token, cookie = "appmsg_token", "cookie"
    article_url = "http://mp.weixin.qq.com/s?__biz=MjM5NDU4ODI0NQ==&mid=2650949647&idx=1&sn=854714295ceee7943fe9426ab10453bf&chksm=bd739b358a041223833057cc3816f9562999e748904f39b166ee2178ce1a565e108fe364b920#rd'"
    test = ArticlesInfo(appmsg_token, cookie)
    comments = test.comments(article_url)
    read_num, like_num, old_like_num = test.read_like_nums(article_url)
    print("comments:")
    pprint(comments)
    print("read_like_num:", read_num, like_num, old_like_num)



# coding: utf-8
import os
from pprint import pprint
from wechatarticles import Url2Html

if __name__ == "__main__":
    # 微信文章下载为离线HTML（含图片）
    # 微信文章的url
    url = "https://mp.weixin.qq.com/s?__biz=MzUzNjk1NDIyNg==&mid=2247506248&idx=1&sn=c9a8b4d2e11fd1b6ae924877da3f0ea8&chksm=faeccd55cd9b44432ea81578b1e33db2ae5ce938ed5a90e32847cb0ea88916a16ac47c92e135&scene=132#wechat_redirect"
    uh = Url2Html()
    # 请提前创建一个以公众号为名的文件夹，并且在该文件夹下创建imgs文件夹
    res = uh.run(url, mode=4, img_path="./黑马青年/imgs")
    print(res)



# coding: utf-8
import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="wechatarticles",
    version="0.6.6",
    author="wnma3mz",
    author_email="wnma3mz@gmail.com",
    description="wechat articles scrapy",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/wnma3mz/wechat_articles_spider",
    packages=setuptools.find_packages(),
    install_requires=["requests>=2.20.0", "beautifulsoup4>=4.7.1"],
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ),
)



# coding:utf-8
import logging
import os
import select
import zlib

import chardet
import time
from http.client import HTTPResponse
from http.server import BaseHTTPRequestHandler, HTTPServer
from socketserver import ThreadingMixIn
from urllib.parse import urlparse, ParseResult, urlunparse
from tempfile import gettempdir

from ssl import wrap_socket, SSLError
from socket import socket
from OpenSSL.crypto import (
    load_certificate,
    FILETYPE_PEM,
    TYPE_RSA,
    PKey,
    X509,
    X509Extension,
    dump_privatekey,
    dump_certificate,
    load_privatekey,
    X509Req,
)

"""
该文件并未项目核心功能，仅作辅助脚本使用
感谢baseproxy项目。原项目地址https://github.com/qiyeboy/BaseProxy，具体详细操作方法请见原项目，步骤简述如下
    1. 设置代理，开启服务
    2. 下载证书http://baseproxy.ca/，并安装至本机(受信任的根证书颁发机构)
    3. 配置完成，每次使用之前开启系统代理即可
为方便使用，在代码上进行了一定的修改，大部分内容来源于，https://github.com/qiyeboy/BaseProxy/blob/master/baseproxy/proxy.py
    1. 修改部分函数名，变量名，函数，尽量保持与mitmproxy一致
    2. 由于项目主要目的是拦截，并非篡改，增加了对链接过滤功能。若链接不包含相关字符，则不做操作，直接返回。ProxyHandle中的`self.filter_url_lst`


二次引用，若有冒犯原作者之处，敬请指出，将删除该文件。
"""

__all__ = [
    "CAAuth",
    "ProxyHandle",
    "ReqIntercept",
    "RspIntercept",
    "MitmProxy",
    "AsyncMitmProxy",
    "Request",
    "Response",
]
logging.basicConfig(
    level=logging.INFO,
    format="[%(asctime)s] %(levelname)s %(message)s",
    datefmt="%Y-%m-%d %H:%M:%S",
)


class HttpTransfer(object):
    version_dict = {9: "HTTP/0.9", 10: "HTTP/1.0", 11: "HTTP/1.1"}

    def __init__(self):
        self.hostname = None
        self.port = None

        self.url = ""  # 无协议头
        # 这是请求
        self.command = None
        self.path = None
        self.request_version = None

        # 这是响应
        self.response_version = None
        self.status = None
        self.reason = None

        self._headers = None

        self._body = b""

    def parse_headers(self, headers_str):
        """
        暂时用不到
        :param headers:
        :return:
        """
        header_list = headers_str.rstrip("\r\n").split("\r\n")
        headers = {}
        for header in header_list:
            [key, value] = header.split(": ")
            headers[key.lower()] = value
        return headers

    def to_data(self):
        raise NotImplementedError("function to_data need override")

    def set_headers(self, headers):
        headers_tmp = {}
        for k, v in headers.items():
            headers_tmp[k.lower()] = v
        self._headers = headers_tmp

    def build_headers(self):
        """
        返回headers字符串
        :return:
        """
        header_str = ""
        for k, v in self._headers.items():
            header_str += k + ": " + v + "\r\n"

        return header_str

    def get_header(self, key):
        if isinstance(key, str):
            return self._headers.get(key.lower(), None)
        raise Exception("parameter should be str")

    @property
    def headers(self):
        """
        获取头部信息
        :return:
        """
        return self._headers

    def set_header(self, key, value):
        """
        设置头部
        :param key:
        :param value:
        :return:
        """
        if isinstance(key, str) and isinstance(value, str):
            self._headers[key.lower()] = value
            return
        raise Exception("parameter should be str")

    def get_body_data(self):
        """
        返回是字节格式的body内容
        :return:
        """
        return self._body

    def set_body_data(self, body):
        if isinstance(body, bytes):
            self._body = body
            self.set_header("Content-length", str(len(body)))
            return
        raise Exception("parameter should be bytes")


class Request(HttpTransfer):
    def __init__(self, req):
        HttpTransfer.__init__(self)

        self.hostname = req.hostname
        self.port = req.port
        # 这是请求
        self.command = req.command
        self.path = req.path
        self.request_version = req.request_version

        self.url = req.hostname + req.path

        self.set_headers(req.headers)

        if self.get_header("Content-Length"):
            self.set_body_data(req.rfile.read(int(self.get_header("Content-Length"))))

    def to_data(self):
        # Build request
        req_data = "%s %s %s\r\n" % (self.command, self.path, self.request_version)
        # Add headers to the request
        req_data += "%s\r\n" % self.build_headers()
        req_data = req_data.encode("utf-8")
        req_data += self.get_body_data()
        return req_data


class Response(HttpTransfer):
    def __init__(self, request, proxy_socket):

        HttpTransfer.__init__(self)

        self.request = request

        h = HTTPResponse(proxy_socket)
        h.begin()
        ##HTTPResponse会将所有chunk拼接到一起，因此会直接得到所有内容，所以不能有Transfer-Encoding
        del h.msg["Transfer-Encoding"]
        del h.msg["Content-Length"]

        self.response_version = self.version_dict[h.version]
        self.status = h.status
        self.reason = h.reason
        self.set_headers(h.msg)

        body_data = self._decode_content_body(
            h.read(), self.get_header("Content-Encoding")
        )
        self.set_body_data(body_data)
        self._text()  # 尝试将文本进行解码

        h.close()
        proxy_socket.close()

    def _text(self):
        body_data = self.get_body_data()
        if self.get_header("Content-Type") and (
            "text" or "javascript"
        ) in self.get_header("Content-Type"):
            self.decoding = chardet.detect(body_data)["encoding"]  # 探测当前的编码
            if self.decoding:
                try:
                    self._body_str = body_data.decode(self.decoding)  # 请求体
                except Exception as e:
                    self._body_str = body_data
                    self.decoding = None
            else:
                self._body_str = body_data
        else:
            self._body_str = body_data
            self.decoding = None

    def get_text(self, decoding=None):
        if decoding:
            return self.get_body_data().decode(decoding)
        return self.get_body_data().decode()

    def set_body_str(self, body_str, encoding=None):
        if isinstance(body_str, str):
            if encoding:
                self.set_body_data(body_str.encode(encoding))
            else:
                self.set_body_data(
                    body_str.encode(self.decoding if self.decoding else "utf-8")
                )
            self._body_str = body_str
            return
        raise Exception("parameter should be str")

    def _encode_content_body(self, text, encoding):

        if encoding == "identity":
            data = text
        elif encoding in ("gzip", "x-gzip"):

            gzip_compress = zlib.compressobj(9, zlib.DEFLATED, zlib.MAX_WBITS | 16)
            data = gzip_compress.compress(text) + gzip_compress.flush()

        elif encoding == "deflate":
            data = zlib.compress(text)
        else:
            data = text

        return data

    def _decode_content_body(self, data, encoding):
        if encoding == "identity":  # 没有压缩
            text = data

        elif encoding in ("gzip", "x-gzip"):  # gzip压缩
            text = zlib.decompress(data, 16 + zlib.MAX_WBITS)
        elif encoding == "deflate":  # zip压缩
            try:
                text = zlib.decompress(data)
            except zlib.error:
                text = zlib.decompress(data, -zlib.MAX_WBITS)
        else:
            text = data

        self.set_header("Content-Encoding", "identity")  # 没有压缩
        return text

    def to_data(self):

        res_data = "%s %s %s\r\n" % (self.response_version, self.status, self.reason)
        res_data += "%s\r\n" % self.build_headers()
        res_data = res_data.encode(self.decoding if self.decoding else "utf-8")
        res_data += self.get_body_data()
        return res_data


class CAAuth(object):
    """
    用于CA证书的生成以及代理证书的自签名
    """

    def __init__(self, ca_file="ca.pem", cert_file="ca.crt"):
        self.ca_file_path = ca_file
        self.cert_file_path = cert_file
        self._gen_ca()  # 生成CA证书，需要添加到浏览器的合法证书机构中

    def _gen_ca(self, again=False):
        # Generate key
        # 如果证书存在而且不是强制生成，直接返回证书信息
        if (
            os.path.exists(self.ca_file_path)
            and os.path.exists(self.cert_file_path)
            and not again
        ):
            self._read_ca(self.ca_file_path)  # 读取证书信息
            return
        self.key = PKey()
        self.key.generate_key(TYPE_RSA, 2048)
        # Generate certificate
        self.cert = X509()
        self.cert.set_version(2)
        self.cert.set_serial_number(1)
        self.cert.get_subject().CN = "baseproxy"
        self.cert.gmtime_adj_notBefore(0)
        self.cert.gmtime_adj_notAfter(315360000)  # 十年
        self.cert.set_issuer(self.cert.get_subject())
        self.cert.set_pubkey(self.key)
        self.cert.add_extensions(
            [
                X509Extension(b"basicConstraints", True, b"CA:TRUE, pathlen:0"),
                X509Extension(b"keyUsage", True, b"keyCertSign, cRLSign"),
                X509Extension(
                    b"subjectKeyIdentifier", False, b"hash", subject=self.cert
                ),
            ]
        )
        self.cert.sign(self.key, "sha256")
        with open(self.ca_file_path, "wb+") as f:
            f.write(dump_privatekey(FILETYPE_PEM, self.key))
            f.write(dump_certificate(FILETYPE_PEM, self.cert))

        with open(self.cert_file_path, "wb+") as f:
            f.write(dump_certificate(FILETYPE_PEM, self.cert))

    def _read_ca(self, file):
        self.cert = load_certificate(FILETYPE_PEM, open(file, "rb").read())
        self.key = load_privatekey(FILETYPE_PEM, open(file, "rb").read())

    def __getitem__(self, cn):
        # 将为每个域名生成的服务器证书，放到临时目录中
        cache_dir = gettempdir()
        root_dir = os.path.join(cache_dir, "baseproxy")
        if not os.path.exists(root_dir):
            os.makedirs(root_dir)

        cnp = os.path.join(root_dir, "baseproxy_{}.pem".format(cn))

        if not os.path.exists(cnp):
            self._sign_ca(cn, cnp)
        return cnp

    def _sign_ca(self, cn, cnp):
        # 使用合法的CA证书为代理程序生成服务器证书
        # create certificate
        try:

            key = PKey()
            key.generate_key(TYPE_RSA, 2048)

            # Generate CSR
            req = X509Req()
            req.get_subject().CN = cn
            req.set_pubkey(key)
            req.sign(key, "sha256")

            # Sign CSR
            cert = X509()
            cert.set_version(2)
            cert.set_subject(req.get_subject())
            cert.set_serial_number(self.serial)
            cert.gmtime_adj_notBefore(0)
            cert.gmtime_adj_notAfter(31536000)  # 一年
            cert.set_issuer(self.cert.get_subject())
            ss = ("DNS:%s" % cn).encode(encoding="utf-8")

            cert.add_extensions([X509Extension(b"subjectAltName", False, ss)])

            cert.set_pubkey(req.get_pubkey())
            cert.sign(self.key, "sha256")

            with open(cnp, "wb+") as f:
                f.write(dump_privatekey(FILETYPE_PEM, key))
                f.write(dump_certificate(FILETYPE_PEM, cert))
        except Exception as e:
            raise Exception("generate CA fail:{}".format(str(e)))

    @property
    def serial(self):
        return int("%d" % (time.time() * 1000))


class ProxyHandle(BaseHTTPRequestHandler):
    def __init__(self, request, client_addr, server):
        self.is_connected = False
        self.hook_init()
        BaseHTTPRequestHandler.__init__(self, request, client_addr, server)

    def hook_init(self):
        # 增加初始化的其他操作，如初始化filter_url_lst
        self.filter_url_lst = []

    def do_CONNECT(self):
        """
        处理https连接请求
        :return:
        """

        self.is_connected = True  # 用来标识是否之前经历过CONNECT
        if self.server.https:
            self.connect_intercept()
        else:
            self.connect_relay()

    def do_GET(self):
        """
        处理GET请求
        :return:
        """
        if self.path == "http://baseproxy.ca/":
            self._send_ca()
            return

        if not self.is_connected:
            # 如果不是https，需要连接http服务器
            try:
                self._proxy_to_dst()
            except Exception as e:
                self.send_error(500, "{} connect fail ".format(self.hostname))
                return
        # 这里就是代理发送请求，并接收响应信息
        request = Request(self)
        # 增加过滤模块
        flag = False
        for filter_x in self.filter_url_lst:
            if filter_x in request.url:
                flag = True
                break
        if flag:
            request = self.mitm_request(request)

        if request:
            self._proxy_sock.sendall(request.to_data())
            # 将响应信息返回给客户端
            response = Response(request, self._proxy_sock)
            if flag:
                response = self.mitm_response(response)

            if response:
                self.request.sendall(response.to_data())
            else:
                self.send_error(404, "response is None")
        else:
            self.send_error(404, "request is None")

    do_HEAD = do_GET
    do_POST = do_GET
    do_PUT = do_GET
    do_DELETE = do_GET
    do_OPTIONS = do_GET

    def _proxy_to_ssldst(self):
        """
        代理连接https目标服务器
        :return:
        """
        ##确定一下目标的服务器的地址与端口

        # 如果之前经历过connect
        # CONNECT www.baidu.com:443 HTTP 1.1
        self.hostname, self.port = self.path.split(":")
        self._proxy_sock = socket()
        self._proxy_sock.settimeout(10)
        self._proxy_sock.connect((self.hostname, int(self.port)))
        # 进行SSL包裹
        self._proxy_sock = wrap_socket(self._proxy_sock)

    def _proxy_to_dst(self):
        # 代理连接http目标服务器
        # http请求的self.path 类似http://www.baidu.com:80/index.html
        u = urlparse(self.path)
        if u.scheme != "http":
            raise Exception("Unknown scheme %s" % repr(u.scheme))
        self.hostname = u.hostname
        self.port = u.port or 80
        # 将path重新封装，比如http://www.baidu.com:80/index.html会变成 /index.html
        self.path = urlunparse(
            ParseResult(
                scheme="",
                netloc="",
                params=u.params,
                path=u.path or "/",
                query=u.query,
                fragment=u.fragment,
            )
        )
        self._proxy_sock = socket()
        self._proxy_sock.settimeout(10)
        self._proxy_sock.connect((self.hostname, int(self.port)))

    def connect_intercept(self):
        """
        需要解析https报文,包装socket
        :return:
        """
        try:
            # 首先建立和目标服务器的链接
            self._proxy_to_ssldst()
            # 建立成功后,proxy需要给client回复建立成功
            self.send_response(200, "Connection established")
            self.end_headers()

            # 这个时候需要将客户端的socket包装成sslsocket,这个时候的self.path类似www.baidu.com:443，根据域名使用相应的证书
            self.request = wrap_socket(
                self.request,
                server_side=True,
                certfile=self.server.ca[self.path.split(":")[0]],
            )
        except SSLError:
            self.send_error(500, "更新证书!")
            return
        except Exception as e:
            self.send_error(500, str(e))
            return

        self.setup()
        self.ssl_host = "https://%s" % self.path
        try:
            self.handle_one_request()
        except Exception as e:
            return

    def connect_relay(self):
        """
        对于https报文直接转发
        """

        self.hostname, self.port = self.path.split(":")
        try:
            self._proxy_sock = socket()
            self._proxy_sock.settimeout(10)
            self._proxy_sock.connect((self.hostname, int(self.port)))
        except Exception as e:
            self.send_error(500)
            return

        self.send_response(200, "Connection Established")
        self.end_headers()

        inputs = [self.request, self._proxy_sock]

        while True:
            readable, writeable, errs = select.select(inputs, [], inputs, 10)
            if errs:
                break
            for r in readable:
                data = r.recv(8092)
                if data:
                    if r is self.request:
                        self._proxy_sock.sendall(data)
                    elif r is self._proxy_sock:
                        self.request.sendall(data)
                else:
                    break
        self.request.close()
        self._proxy_sock.close()

    def _send_ca(self):
        # 发送CA证书给用户进行安装并信任
        cert_path = self.server.ca.cert_file_path
        with open(cert_path, "rb") as f:
            data = f.read()

        self.send_response(200)
        self.send_header("Content-Type", "application/x-x509-ca-cert")
        self.send_header("Content-Length", len(data))
        self.send_header("Connection", "close")
        self.end_headers()
        self.wfile.write(data)

    def mitm_request(self, req):
        for p in self.server.req_plugs:
            req = p(self.server).deal_request(req)
        return req

    def mitm_response(self, rsp):
        for p in self.server.rsp_plugs:
            rsp = p(self.server).deal_response(rsp)
        return rsp


class MitmProxy(ThreadingMixIn, HTTPServer):
    def __init__(
        self,
        server_addr=("", 8080),
        RequestHandlerClass=ProxyHandle,
        bind_and_activate=True,
        https=True,
        ca_file="ca.pem",
        cert_file="ca.crt",
    ):
        HTTPServer.__init__(self, server_addr, RequestHandlerClass, bind_and_activate)
        logging.info(
            "HTTPServer is running at address( %s , %d )......"
            % (server_addr[0], server_addr[1])
        )
        self.req_plugs = []  ##请求拦截插件列表
        self.rsp_plugs = []  ##响应拦截插件列表
        self.ca = CAAuth(ca_file=ca_file, cert_file=cert_file)
        self.https = https

    def register(self, intercept_plug):
        if not issubclass(intercept_plug, InterceptPlug):
            raise Exception(
                "Expected type InterceptPlug got %s instead" % type(intercept_plug)
            )

        if issubclass(intercept_plug, ReqIntercept):
            self.req_plugs.append(intercept_plug)

        if issubclass(intercept_plug, RspIntercept):
            self.rsp_plugs.append(intercept_plug)


class AsyncMitmProxy(MitmProxy):
    pass


class InterceptPlug(object):
    def __init__(self, server):
        self.server = server


class ReqIntercept(InterceptPlug):
    def deal_request(self, request):
        pass


class RspIntercept(InterceptPlug):
    def deal_response(self, response):
        pass
